const lo = require('lodash');
const consts = require('../../consts');
const hlpPolicy = require('../../helpers_policy');
const apiUtils = require('../../common/open_api');

const TaskTypeSettingsValue = 'PluginTaskType';
const taskConverters = {
    AntiCryptor: require('./types/AntiCryptor.js'),
    AutoRuleGenerator: require('./types/AutoRuleGenerator.js'),
    CheckAppIntegrity: require('./types/CheckAppIntegrity.js'),
    CheckFilesIntegrity: require('./types/CheckFilesIntegrity.js'),
    DeviceControl: require('./types/DeviceControl.js'),
    DeviceRuleGenerator: require('./types/DeviceRuleGenerator.js'),
    FileIntegrityMonitor: require('./types/FileIntegrityMonitor.js'),
    Firewall: require('./types/Firewall.js'),
    // HostBlocker
    ICAPStorageProtection: require('./types/ICAPStorageProtection.js'),
    InstallKey: require('./types/InstallKey.js'),
    KSNService: require('./types/KSNService.js'),
    LogInspector: require('./types/LogInspector.js'),
    NetAppAntiCryptor: require('./types/NetAppAntiCryptor.js'),
    NetAppMonitor: require('./types/NetAppMonitor.js'),
    NetworkAttackBlocker: require('./types/NetworkAttackBlocker.js'),
    OnAccess: require('./types/OnAccess.js'),
    OnDemand: require('./types/OnDemand.js'),
    // PlcCheck
    // PlcInvestigate
    ProgramExecutionPrevention: require('./types/ProgramExecutionPrevention.js'),
    RetranslateUpdates: require('./types/RetranslateUpdates.js'),
    ScanQuarantine: require('./types/ScanQuarantine.js'),
    ScriptChecker: require('./types/ScriptChecker.js'),
    UpdateBases: require('./types/UpdateBases.js'),
    UpdateComponents: require('./types/UpdateComponents.js'),
    WebTrafficMonitor: require('./types/WebTrafficMonitor.js')
    // WiFiControl
}

function LocalTaskSettings () {
    let curHostId = '';
    let curTaskId = '';
    let groupId = 0;
    let currentTaskType = '';
    let taskAdditionalParams = {};
    let policiesCache = {};
    let GroupTask = false;
    let LockTask = false;

    const GetTempDataName = () => {
        return 'localTaskCache'+ curTaskId;
    }

    async function CacheFromTempDataStore(tempDataStore) {
        if(tempDataStore!==undefined) {
            const temp = await tempDataStore.get(GetTempDataName());
            if(temp!==undefined)
                taskAdditionalParams = temp;
        }
    }

    async function CacheToTempDataStore(tempDataStore) {
        if(tempDataStore!==undefined) {
            await tempDataStore.set(GetTempDataName(), taskAdditionalParams);
        }
    }

    async function getTaskDataCache(kscApi, taskId, hostId) {
        let result = await hlpPolicy.GetTaskData(kscApi, taskId, hostId);
        runtime.logger.log(consts.logProductPref + ' getTaskDataCache: Result = ' + + JSON.stringify(result));
        if ( result && result.TASK_ADDITIONAL_PARAMS !==undefined ) {
            taskAdditionalParams = lo.clone(result.TASK_ADDITIONAL_PARAMS)
            currentTaskType = result.TASK_NAME;
            GroupTask = (result.TASK_INFO_PARAMS!==undefined && result.TASK_INFO_PARAMS.PRTS_TASK_GROUPID!=undefined) ? true : false;
        }
        else{
            curTaskId = '';
            currentTaskType = '';
            taskAdditionalParams = {};
            GroupTask = false;
            LockTask = false;
        }
    }

    async function UpdateRbacState(api, task_type) {
        LockTask = false;

        try{
            const ReadOnly  = await api.HstAccessControl.IsTaskTypeReadonly({
                lGroupId: groupId,
                szwProduct: consts.wstrProduct,
                szwVersion: consts.wstrVersion,
                szwTaskTypeName: task_type
            });
            if(ReadOnly.result!==undefined && ReadOnly.result.PxgError===undefined){
                LockTask = ReadOnly.result;
                runtime.logger.log(consts.logProductPref + ' UpdateRbacState: LockTask = ' + LockTask);
            }
        }
        catch (error) {
            console.error(error);
        }
    }

    const UpdateTaskTypeByForm = (taskType, form) => {
        if(taskType === 'policy'){
            switch (form) {
                case 'Protection_KsnCommon':
                case 'Protection_KsnEula':
                case 'Protection_KpsnEula':
                case 'Protection_KmpEula':
                    return 'KsnService';
                default:
                    break
            }
        }
        return taskType;
    }

    /*
     * Called via api.js in getFormModel
     */
    async function getTaskData(kscApi, taskId, hostId, taskType, form, sGroupId, tempDataStore) {
        taskType = UpdateTaskTypeByForm(taskType, form);
        runtime.logger.log(consts.logProductPref + ' getTaskData: Task = ' + taskType + ' Form = ' + form);

        if(curHostId!==hostId || curTaskId!= taskId) {
            curHostId = hostId;
            curTaskId = taskId;
            groupId = parseInt(sGroupId);
            LockTask = false;
            GroupTask = false;
            policiesCache = {};
            taskAdditionalParams = {};
            currentTaskType = '';
            runtime.logger.log(consts.logProductPref + ' Reset data  Task = ' + taskType + ' Form = ' + form);
        }

        await getTaskDataCache(kscApi, taskId, hostId);

        await UpdateRbacState(kscApi, currentTaskType);

        let computedData = {};
        const converter = taskConverters[currentTaskType];

        // request from Root task page form with TaskType
        if(converter){
            const activePolicyId = await hlpPolicy.GetHostActivePolicyId(kscApi, hostId);
            runtime.logger.log('KSWS: getTaskData: activePolicyId = ' + activePolicyId);
            policiesCache = {};

            if(activePolicyId===-1 || activePolicyId===undefined){
                hlpPolicy.UpdatePoliciesCacheAsDefault(converter.defPolicySections, policiesCache);
                runtime.logger.log('Default policy data: ' + JSON.stringify(policiesCache));
            } else {
                await hlpPolicy.UpdatePoliciesCache(kscApi, activePolicyId, converter.defPolicySections, policiesCache);
                runtime.logger.log('Active policy data: ' + JSON.stringify(policiesCache));
            }
            if(LockTask && converter.RbacToPolicyCache!==undefined){
                policiesCache = converter.RbacToPolicyCache(policiesCache);
                runtime.logger.log('Policy data with Rbac: ' + JSON.stringify(policiesCache));
            }

            // get data from cache
            try {
                if(converter.toViewEx!==undefined){
                    computedData = await converter.toViewEx(kscApi, taskAdditionalParams, policiesCache, form, GroupTask, LockTask);
                } else{
                    const bIsHosted = await apiUtils.IsSetHostedMode(kscApi);
                    computedData = converter.toView(taskAdditionalParams, policiesCache, form, GroupTask, LockTask,bIsHosted);
                }
                computedData[TaskTypeSettingsValue] = currentTaskType;
                await CacheToTempDataStore(tempDataStore);

            } catch(error){
                runtime.logger.error(consts.logProductPref + ' toView: ' + error);
            }
        } else {
            runtime.logger.error(consts.logProductPref + ' Unknown task type = ' + currentTaskType + '! Cant get data ');
        }

        runtime.logger.log('KSWS: getTaskData end. Task = ' + taskType + 'Data: ' + JSON.stringify(computedData));
        return computedData;
    }

    async function getTaskAdditionalParams(TaskType, tempDataStore) {
        console.log('ksws_windows:LocalTask:getTaskAdditionalParams ' + TaskType + ', Params:' + JSON.stringify(taskAdditionalParams));
  //      await CacheFromTempDataStore(tempDataStore);
        return taskAdditionalParams;
    }

    async function updateTaskData ({kscApi, modelData, formType, hostId, tDataStore}) {
        let converter = taskConverters[currentTaskType];
        if (converter) {
   //         await CacheFromTempDataStore(tDataStore);
            if(converter.toRawEx!==undefined){
                await converter.toRawEx(kscApi, taskAdditionalParams, modelData, formType, hostId);
            }
            else{
                converter.toRaw(taskAdditionalParams, modelData, formType);
            }
     //       await CacheToTempDataStore(tDataStore);
        }
    }

    return {
        getTaskData,
        getTaskAdditionalParams,
        updateTaskData
    }
}

module.exports = LocalTaskSettings
