const hlp = require('../../helpers')
const hlpPep = require('../../helpers_pep')

const PolicyRulesUsageStrategy = { AddToLocal: 1, ReplaceLocal: 2 }
const OriginNamesByType = { 0:'', 1:'Default', 2:'User', 3:'Auto' }
const TargetNamesByType = {1:'Binary', 2:'ScriptOrMsi'}
const ActionNamesByType = {1:'Allow', 2:'Deny',3:'Audit'}
const ConditionTypeEnum = {'Certificate': 1, 'Sha256': 2, 'Path': 3 }

const UiRuleConverter = {
    1: {
        getRule : function(uiRule) {
            let out = {
                "$": GetRuleAttributes(uiRule),
                "CertificateRuleCondition": [GetCertRuleCondition(uiRule.CertificateRuleCondition)]
            }
            if(uiRule.Exclusions.length)
                out.Exclusions = GetRuleExclusions(uiRule.Exclusions);
            return out;
        },
        getExclusion : function(uiExcl) {
            return GetCertRuleCondition(uiExcl.CertificateRuleCondition)
        }
    },
    2:{
        getRule : function(uiRule) {
            let out = {
                "$": GetRuleAttributes(uiRule),
                "HashRuleCondition": [{ "$": { "Hash": uiRule.HashRuleCondition.Hash } }]
            }
            if(uiRule.Exclusions.length)
                out.Exclusions = GetRuleExclusions(uiRule.Exclusions);
            return out;
        },
        getExclusion : function(uiExcl) {
            return {"$":{"Type":"SHA256","Hash":uiExcl.HashRuleCondition.Hash}}
        }
    },
    3:{
        getRule : function(uiRule) {
            let out = {
                "$": GetRuleAttributes(uiRule),
                "PathRuleCondition": [{ "$": { "Path": uiRule.PathRuleCondition.Path } }]
            }
            if(uiRule.Exclusions.length)
                out.Exclusions = GetRuleExclusions(uiRule.Exclusions);
            return out;
        },
        getExclusion : function(uiExcl) {
            return {"$":{"Path":uiExcl.PathRuleCondition.Path}}
        }
    }
}

const GetRuleAttributes = (uiRule) => {
    return {
        "Name": uiRule.Name,
        "Description": "",
        "Origin": OriginNamesByType[uiRule.Origin],
        "Target": TargetNamesByType[uiRule.Target],
        "UserOrGroupSid": uiRule.UserOrGroupSid,
        "Action": ActionNamesByType[uiRule.Action]
    }
}

const GetCertRuleCondition = (obj) => {
    return {"$":{
            "UseSubject": obj.UseSubject ? "true" : "false",
            "Subject": obj.Subject,
            "UseThumbprint": obj.UseThumbprint ? "true" : "false",
            "Thumbprint": obj.Thumbprint
        } };
}

const GetRuleExclusions = (uiExcl) => {
    let CertExcl = [];
    let HashExcl = [];
    let PathExcl = [];

    for(let item of uiExcl){
        const outExcl = UiRuleConverter[item.RuleCondition].getExclusion(item)
        if(item.RuleCondition===ConditionTypeEnum.Certificate){
            CertExcl.push(outExcl);
        }
        else if(item.RuleCondition===ConditionTypeEnum.Sha256){
            HashExcl.push(outExcl);
        }
        else if(item.RuleCondition===ConditionTypeEnum.Path){
            PathExcl.push(outExcl);
        }
    }

    let excl = {};
    if(CertExcl.length){
        excl.CertificateRuleCondition = CertExcl;
    }
    if(HashExcl.length){
        excl.HashRuleCondition = HashExcl;
    }
    if(PathExcl.length){
        excl.PathRuleCondition = PathExcl;
    }

    return [excl];
}

const getDummyRulesTemplate = (jsonRules) => {
    const ver = (jsonRules && jsonRules.ApplicationControl && jsonRules.ApplicationControl.$ && jsonRules.ApplicationControl.$.Version) ?
        jsonRules.ApplicationControl.$.Version : '1';
    return { ApplicationControl: { '$': {Version: ver}, 'RuleCollection': [] } }
}

const toJsonRules = (uiRules, jsonTemplate) => {
    let jsonRules = jsonTemplate;

    let CertRule = [];
    let HashRule = [];
    let PathRule = [];

    for(let rule of uiRules){
        const outRule = UiRuleConverter[rule.RuleCondition].getRule(rule)

        if(rule.RuleCondition===ConditionTypeEnum.Certificate){
            CertRule.push(outRule);
        }
        else if(rule.RuleCondition===ConditionTypeEnum.Sha256){
            HashRule.push(outRule);
        }
        else if(rule.RuleCondition===ConditionTypeEnum.Path){
            PathRule.push(outRule);
        }
    }
    let collect = {};
    if(CertRule.length){
        collect.CertificateRule = CertRule;
    }
    if(HashRule.length){
        collect.HashRule = HashRule;
    }
    if(PathRule.length){
        collect.PathRule = PathRule;
    }
    jsonRules.ApplicationControl.RuleCollection = [collect];
    return jsonRules;
}

const toViewRules = (data) => {
    let jsonRules = {};
    let dummyRules = '';
    if (data.RuleUsageStrategy.KLPRSS_Val[0] === PolicyRulesUsageStrategy.ReplaceLocal) {
        jsonRules = hlp.parseBinXMLtoJSON(data.RuleCollection.KLPRSS_Val[0].value);
        dummyRules = data.AdditionalRuleCollection.KLPRSS_Val[0].value;
    } else {
        jsonRules = hlp.parseBinXMLtoJSON(data.AdditionalRuleCollection.KLPRSS_Val[0].value);
        dummyRules = data.RuleCollection.KLPRSS_Val[0].value;
    }

    return {
        Rules: hlpPep.toPepUiRiles(jsonRules),
        jsonTempl: getDummyRulesTemplate(jsonRules),
        dummyTempl: dummyRules
    }
}

const toViewSettingsRules = (binData) => {
    let jsonRules = hlp.parseBinXMLtoJSON(binData);
    return {
        Rules: hlpPep.toPepUiRiles(jsonRules),
        jsonTempl: getDummyRulesTemplate(jsonRules)
    }
}

const toRowRules = (changedData) => {
    let jsonRules = toJsonRules(changedData.uiFields.Rules, changedData.uiFields.jsonTempl)
    let packedRules = hlp.JSONtoBinXML(jsonRules);

    const bRulesMandatoried = changedData.RuleUsageStrategy.KLPRSS_Mnd ? true : false
    // if Rules are mandatoried and set strategy AddToLocal we should save rules in additional_rule_collection
    if(changedData.RuleUsageStrategy.KLPRSS_Val[0]===PolicyRulesUsageStrategy.AddToLocal)
    {
        // in this case we save customized copy of policy - to allow father edition in UI without closing this dialog (BF: #2134690)
        changedData.AdditionalRuleCollection.KLPRSS_Mnd = bRulesMandatoried;
        changedData.AdditionalRuleCollection.KLPRSS_Val[0].value = packedRules;
        changedData.RuleCollection.KLPRSS_Mnd = false;
        changedData.RuleCollection.KLPRSS_Val[0].value = changedData.uiFields.dummyTempl;
    }
    else if(changedData.RuleUsageStrategy.KLPRSS_Val[0]===PolicyRulesUsageStrategy.ReplaceLocal)
    {
        changedData.RuleCollection.KLPRSS_Mnd = bRulesMandatoried;
        changedData.RuleCollection.KLPRSS_Val[0].value = packedRules;
        changedData.AdditionalRuleCollection.KLPRSS_Mnd = bRulesMandatoried;
        changedData.AdditionalRuleCollection.KLPRSS_Val[0].value = changedData.uiFields.dummyTempl;
    }
    delete changedData.uiFields;
    return changedData;
}

const toRowSettingsRules = (changedData) => {
    const jsonRules = toJsonRules(changedData.uiFields.Rules, changedData.uiFields.jsonTempl)
    return [{
        type: 'binary',
        value: hlp.JSONtoBinXML(jsonRules)
    }];
}

const GetDummyBinRules = () =>{
    return [{
        type: 'binary',
        value: 'PEFwcGxpY2F0aW9uQ29udHJvbCBWZXJzaW9uPSIxIj48UnVsZUNvbGxlY3Rpb24gLz48L0FwcGxpY2F0aW9uQ29udHJvbD4='
    }];
}

module.exports = {
    toViewRules,
    toViewSettingsRules,
    toRowRules,
    toRowSettingsRules,
    GetDummyBinRules
}